#!/apps/public/solaris/bin/perl 

my %big5array;
my %englisharray;
my $no_translation = "(translation unavailable)";

my $dict = "big5dict";
my $index = "big5dict.index";

while (<>) {

    my $syllable;
    my $characters;
    my $english;
    my $onechar;

    if (/^([a-z][a-z]*[0-5])\t(.*)$/) {
	$syllable = $1;
	$characters = $2;
	while ($characters) {
	    $onechar = substr($characters, 0, 2);
	    if (!defined($big5array{$onechar})) {
		$big5array{$onechar} = $syllable;
		$englisharray{$onechar} = $no_translation;
	    }
	    $characters = substr($characters, 2);
	}
    } elsif (m,^([\200-\377][^ ]+)\s+\[(.*?)\]\s+/(.*?)/?\s+$,) {
	$characters = $1;
	$syllable = $2;
	$english = $3;
	$syllable =~ s/ //g;
	$english =~ s/[\'\"\`]//g;
	$english =~ s./., .g;
	if (defined($big5array{$characters}) and
	    $big5array{$characters} ne $no_translation) {
	    $big5array{$characters} .= "|" . $syllable;
	    $englisharray{$characters} .= "|" . $english;
	} else {	    
	    $big5array{$characters} = $syllable;
	    $englisharray{$characters} = $english;
	}
    }
}

my @out =
    map {
	"$_ [" . $big5array{$_} . "] /" . $englisharray{$_} . "/\n"
	}
    sort {
	substr($a, 0, 2) cmp substr($b, 0, 2) or
	    length($a) <=> length($b) or
		$a cmp $b
	}
    keys %big5array;

open DICT, ">$dict";
open INDEX, ">$index";

my @idxarray;
my $maxord = 0;
my $minord = 200000;

foreach (@out) {

    my $pos = tell DICT;
    my $idxchar = substr($_, 0, 2);
    my $idxlen = index($_, " ") / 2;

    if ($idxlen > 5) { $idxlen = 5; }
    next if ($idxlen < 1);

    my $idxord =
	(ord(substr($idxchar, 0, 1)) - 128) * 256 +
	    ord(substr($idxchar, 1, 1));
    $idxord = $idxord * 5 + $idxlen - 1;

    if (!defined($idxarray[$idxord])) {
	$idxarray[$idxord] = $pos;
	$maxord = $idxord if ($idxord > $maxord);
	$minord = $idxord if ($idxord < $minord);
    }

    print DICT $_;
}

close DICT;

print INDEX pack "N", $minord;
for ($idxord = $minord; $idxord <= $maxord; $idxord++) {
    if (defined $idxarray[$idxord]) {
	print INDEX pack "N", ($idxarray[$idxord] + 1);
    } else {
	print INDEX pack "N", 0;
    }
}

close INDEX;

